import argparse
import logging
import os
import sys

from motu_utils import motu_api, utils_log

import ecfas.cmems_datasets as dsets
import numpy as np
import pandas as pd

log = logging.getLogger(__name__)


def motu_download(usr, pwd, motu, t_start, t_end, out_dir, data_sets, dset_type, region=None, modtype='fcst', label='analysis'):
    """ Downloads hydro or wave data from CMEMS using motu API """
    log.info(f">> Downloading data for {dset_type}")

    max_time = 1
    tot_time = pd.Timestamp(t_end) - pd.Timestamp(t_start)
    if tot_time.total_seconds() < 0:
        log.error('The total time is negative, something is wrong with the dates')
        sys.exit(1)
    num_time_intervals = np.ceil(tot_time.days / max_time)

    times = []
    for tt in range(0, int(num_time_intervals)):
        times.append([pd.Timestamp(t_start) + pd.Timedelta(days=int(max_time * tt)), pd.Timestamp(t_start) + pd.Timedelta(days=int(max_time * (tt + 1)))])

    setsel = []
    if region is None:
        setsel.append(list(data_sets.keys()))
    elif region == 'ARC' and dset_type == 'PHY':
        setsel.append('ARC_ocean')
        setsel.append('ARC')
    else:
        setsel.append(region)

    for dseti in setsel:
        log.info(f">> Region {dseti}")
        out = os.path.join(out_dir, '%s' % dseti)
        out = os.path.join(out, 'data')

        if not os.path.exists(out):
            os.makedirs(out)

        if 'lon_min' in data_sets[dseti].keys() and 'lat_min' in data_sets[dseti].keys() and np.abs(data_sets[dseti]['lon_max'] - data_sets[dseti]['lon_min']) * np.abs(data_sets[dseti]['lat_max'] - data_sets[dseti]['lat_min']) > 858:
                log.info('WARNING: REQUESTED SPATIAL EXTENT MAY BE TOO BIG FOR SERVER. TRY REDUCING SPATIAL EXTENT')

        for tt in range(0, int(num_time_intervals)):
            query_args = get_query_args(usr, pwd, motu, times[tt][0], times[tt][1], out, data_sets[dseti], dset_type, label)
            if os.path.isfile(os.path.join(out, query_args.out_name)):
                log.info('File ' + query_args.out_name + ' already present')
                continue
            try:
                log.info('Downloading ' + query_args.out_name)
                motu_api.execute_request(query_args)
            except Exception as e:
                log.error('Download failed: ' + str(e))
                sys.exit(1)


def motu_download_satdata(usr, pwd, motu, t_start, t_end, out_dir, label='analysis'):
    """ Downloads satellite data from CMEMS using motu API """

    max_time = 1
    tot_time = pd.Timestamp(t_end) - pd.Timestamp(t_start)
    if tot_time.total_seconds() < 0:
        log.info('The total time is negative, something is wrong with the dates')
        sys.exit(1)
    num_time_intervals = np.ceil(tot_time.days / max_time)

    times = []
    for tt in range(0, int(num_time_intervals)):
        times.append([pd.Timestamp(t_start) + pd.Timedelta(days=int(max_time * tt)), pd.Timestamp(t_start) + pd.Timedelta(days=int(max_time * (tt + 1)))])

    for dseti in dsets.datasets_sat:
        out = os.path.join(out_dir, '%s' % dseti)
        out = os.path.join(out, 'data')

        if not os.path.exists(out):
            os.makedirs(out)

        for tt in range(0, int(num_time_intervals)):
            if type(dsets.datasets_sat[dseti]['product']) is list:
                dset = dsets.datasets_sat[dseti]
                for iprod in dsets.datasets_sat[dseti]['product']:
                    dset['product'] = iprod
                    query_args = get_query_args(usr, pwd, motu, times[tt][0], times[tt][1], out, dset, "", label)
                    if os.path.isfile(os.path.join(out, query_args.out_name)):
                        log.info('File ' + query_args.out_name + ' is already present')
                        continue
                    try:
                        log.info('Downloading ' + query_args.out_name)
                        motu_api.execute_request(query_args)
                    except Exception as e:
                        log.error('Download failed: ' + str(e))
                        sys.exit(1)

            else:
                query_args = get_query_args(usr, pwd, motu, times[tt][0], times[tt][1], out, dsets.datasets_sat[dseti], "", label)
                if os.path.isfile(os.path.join(out, query_args.out_name)):
                    log.info('File ' + query_args.out_name + ' is already present')
                    continue
                try:
                    motu_api.execute_request(query_args)
                except Exception as e:
                    log.error('Download failed: ' + str(e))
                    sys.exit(1)


def get_query_args(usr, pwd, motu, tmin, tmax, out_dir, dset, dset_type, label):
        """ Returns an argparse object with required query arguments and values """
        query_args = argparse.Namespace()
        # Required by API
        query_args.auth_mode = 'cas'
        query_args.size = False
        query_args.sync = False
        query_args.console_mode = False
        query_args.describe = False
        query_args.user_agent = None
        query_args.socket_timeout = None
        query_args.outputWritten = None
        query_args.block_size = 65536

        # Required by API, over written if present
        query_args.proxy_server = None
        query_args.proxy_user = None
        query_args.proxy_pwd = None
        query_args.depth_min = None
        query_args.depth_max = None
        query_args.longitude_min = None
        query_args.longitude_max = None
        query_args.latitude_min = None
        query_args.latitude_max = None

        query_args.motu = motu
        query_args.user = usr
        query_args.pwd = pwd
        query_args.service_id = dset['service']
        query_args.product_id = dset['product']

        if 'lon_min' in dset.keys() and 'lat_min' in dset.keys():
            query_args.longitude_min = str(dset['lon_min'])
            query_args.longitude_max = str(dset['lon_max'])
            query_args.latitude_min = str(dset['lat_min'])
            query_args.latitude_max = str(dset['lat_max'])

        if 'depth_min' in dset.keys():
            # append these attributes
            query_args.depth_min = str(dset['depth_min'])
            query_args.depth_max = str(dset['depth_max'])

        query_args.date_min = str(tmin)
        query_args.date_max = str(tmax)
        tmin_str = str(tmin).replace(':', '-').replace(' ', '_')
        tmax_str = str(tmax).replace(':', '-').replace(' ', '_')
        file_name = label + '_' + tmin_str + '_' + tmax_str + '.nc'

        query_args.variable = dset['var']
        ds_vars = dset['var']
        if ds_vars is None:
            file_name = 'all' + '_' + file_name
        elif isinstance(ds_vars, list):
            if isinstance(ds_vars[0], list):
                file_name = dset_type + 'subset' + '_' + file_name
            else:
                if ds_vars[0] != None:
                    file_name = ds_vars[0] + '_' + file_name
        else:
            file_name = ds_vars + '_' + file_name

        query_args.out_dir = out_dir
        query_args.out_name = file_name

        return query_args


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri May 14 13:25:18 2021
@author: mirazoki
"""

import argparse
import datetime
import glob
import os
import sys

import ecfas.cmems_datasets as dsets
import numpy as np
import xarray as xr
import pandas as pd


def run_checks(outdir, baselinedir, region, start_time):

    t0 = datetime.datetime.strptime(start_time, "%Y%m%d_%H%M%S")
    label = t0.strftime("%Y-%m-%d_%H-%M-%S")
    ts_datadir = os.path.join(outdir, region, 'data')
    tsproc_datadir = os.path.join(outdir, region, 'timeseries')

    pred_start = t0 - datetime.timedelta(days=2)
    pred_end = t0 + datetime.timedelta(days=5)

    print("====================QUALITY CHECK===============\n")
    print("==REGION:%s\n" % region)
    print("==BULLETIN DATE: %s\n" % t0.strftime("%Y%m%d_%H%M%S"))
    print("\n")
    print('Comparing EXTRACTED timeseries...\n')

    ok1 = True
    # find timeseries and compute differences for each variable
    ref_file = glob.glob(os.path.join(ts_datadir, 'tseries_coastal_%s*.nc' % label))[0]
    if not os.path.exists(ref_file):
        print(f'Cannot find reference timeseries file(s) {ref_file}')
        return False

    baseline_file = glob.glob(os.path.join(baselinedir, region, 'data', 'tseries_coastal_%s*.nc' % label))[0]
    if not os.path.exists(ref_file):
        print(f'Cannot find baseline file(s) {baseline_file}')
        return False

    refdata = xr.open_dataset(ref_file, decode_times=False)
    twindata = xr.open_dataset(baseline_file, decode_times=False)

    for vari in twindata.variables:
        # skip non-numeric data (e.g. station names, coordinates will tell us if this is ok)
        if refdata[vari].dtype in ['float32', 'int64']:
            checkdata = np.allclose(refdata[vari].values, twindata[vari].values, equal_nan=True, atol=0.01)
            if checkdata:
                print('Comparison of variable %s:%s\n' % (vari, 'succeded'))
            else:
                print('Comparison of variable %s:%s\n' % (vari, 'failed'))
                checkarr = np.zeros(refdata['stations'].shape, dtype='bool')
                maxdiff = np.zeros(refdata['stations'].shape)
                for istation in np.arange(refdata.dims['stations']):
                    checki = np.allclose(refdata[vari].isel(stations=istation).values, twindata[vari].isel(stations=istation).values, equal_nan=True)
                    checkarr[istation] = checki
                    maxdiff[istation] = np.max(np.abs(refdata[vari].isel(stations=istation).values - twindata[vari].isel(stations=istation).values))
                print('max diff %s: %f [%s] (%s)\n' % (vari, np.max(maxdiff[~np.isnan(maxdiff)]), refdata[vari].units, refdata['stnames'].isel(stations=np.where(maxdiff == np.max(maxdiff[~np.isnan(maxdiff)]))[0]).values))
                ok1 = False

    if ok1:
        print("Evaluation EXTRACTED timeseries: SUCCEDED\n")
    else:
        print("Evaluation EXTRACTED timeseries: FAILED\n")

    #=========================================================================================================
    print('Comparing PROCESSED timeseries...\n')
    ok2 = True
    if region != 'ARC_ocean':
        labelproc = ['b%s_%s_%s' % (t0.strftime('%Y%m%d%H%M%S'), tstarti.strftime('%Y%m%d'), (tstarti + datetime.timedelta(days=1)).strftime('%Y%m%d')) for tstarti in pd.date_range(start=pred_start, end=pred_end, freq='1D', closed='left')]

        for ilab in labelproc:
            print('Checking daily file %s\n' % ilab)
            ref_file = glob.glob(os.path.join(tsproc_datadir, 'TScoast_%s_%s.nc' % (region, ilab)))[0]
            if not os.path.exists(ref_file):
                print(f'Cannot find reference daily file(s) {ref_file}')
                return False

            baseline_file = glob.glob(os.path.join(baselinedir, region, 'timeseries', 'TScoast_%s_%s.nc' % (region, ilab)))[0]
            if not os.path.exists(baseline_file):
                print(f'Cannot find reference file(s) {baseline_file}')
                return False

            refdata = xr.open_dataset(ref_file, decode_times=False)
            twindata = xr.open_dataset(baseline_file, decode_times=False)

            for vari in twindata.variables:
                # skip non-numeric data (e.g. station names, coordinates will tell us if this is ok)
                if refdata[vari].dtype in ['float32', 'int64']:
                    checkdata = np.allclose(refdata[vari].values, twindata[vari].values, equal_nan=True, atol=0.01)
                    if not checkdata:
                        print('Comparison of variable %s:%s\n' % (vari, 'failed'))
                        checkarr = np.zeros(refdata['stations'].shape, dtype='bool')
                        maxdiff = np.zeros(refdata['stations'].shape)
                        for istation in np.arange(refdata.dims['stations']):
                            checki = np.allclose(refdata[vari].isel(stations=istation).values, twindata[vari].isel(stations=istation).values, equal_nan=True)
                            checkarr[istation] = checki
                            maxdiff[istation] = np.max(np.abs(refdata[vari].isel(stations=istation).values - twindata[vari].isel(stations=istation).values))
                        print('max diff %s: %f [%s] (%s)\n' % (vari, np.max(maxdiff[~np.isnan(maxdiff)]), refdata[vari].units, refdata['stnames'].isel(stations=np.where(maxdiff == np.max(maxdiff[~np.isnan(maxdiff)]))[0]).values))
                        ok2 = False
    if ok2:
        print("Evaluation PROCESSED timeseries: SUCCEDED\n")
    else:
        print("Evaluation PROCESSED timeseries: FAILED\n")

    print("====================END========================")
    if not ok1 or not ok2:
        return False
    else:
        return True


def main():
    parser = argparse.ArgumentParser("qual_checks")
    parser.add_argument('-o', '--outputs', metavar='<output_dir>', required=True, help='Absolute path to output data to be checked')
    parser.add_argument('-b', '--baselines', metavar='<baseline_dir>', required=True, help='Absolute path to baseline data to be checked against')
    parser.add_argument('-r', '--region', metavar='<region>', required=True, help='Region of interest, one of ARC, BAL, BS. IBI, MED, NWS, GLO. Defaults to all')
    parser.add_argument('-t', '--t0', metavar='<YYmmdd_HHMMSS>', required=True, help='Start time t0 in the format YYmmdd_HHMMSS')

    args = parser.parse_args()

    ok = run_checks(args.outputs, args.baselines, args.region, args.t0)
    # If any of the checks fail, exit with error flag, which will be picked up by test framework as a fail
    if not ok:
        sys.exit(1)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Feb  3 16:10:11 2021

@author: mirazoki

combining forecasted fields at coastal locations

steps (special treatment for arctic, leave out for now)
in all other locations, we take the PHY coastal points and interpolate the wave quantities at those locations

1. retrieve WLs
2. retrieve waves at those same points. 
-ideally, we just save he weights of the regridding so that this is only done once
-because of nan values, it might be that we have to drop the idea of griddedinterpolant and just use Delaunay. For Delaunay, one can save weights.
-test for speed (interpolating every time between regular grids might give better performance than saving the delaunay interpolation, but Nan performance has to be checked)
-maybe interpolation in time is needed if wave params are outputted at a different time-stamp
"""

import datetime
from functools import partial
import logging
import os

import matplotlib
from scipy import interpolate
from scipy.interpolate import LinearNDInterpolator
from scipy.interpolate import NearestNDInterpolator
from scipy.spatial import Delaunay
from scipy.spatial import cKDTree as KDTree

import ecfas.cmems_datasets as dsets
import ecfas.utils as utils
import multiprocessing as mp
import netCDF4 as nc
import numpy as np
import pandas as pd
import xarray as xr

log = logging.getLogger(__name__)


def saveConcatenatedNC(datadir, PHYvar, label, fperiods, fperiode, filename):

    listfilesWL = np.array([os.path.join(datadir, 'data', '%s_%s_%s_%s.nc' % (PHYvar, label, fperiods[ii].strftime('%Y-%m-%d_%H-%M-%S'), fperiode[ii].strftime('%Y-%m-%d_%H-%M-%S'))) for ii in np.arange(len(fperiods))])

    ds = xr.open_mfdataset(listfilesWL)
    _, index = np.unique(ds['time'], return_index=True)
    ds = ds.isel(time=index)

    path = os.path.join(datadir, "data", filename)
    log.info("Temp file saved in: %s", path)
    ds.to_netcdf(path)


def calc_interp(itime, ncdataWAV, maskarr, coords_coast, tri):
    rav = np.ravel(ncdataWAV[itime])[~maskarr]
    interpolator = LinearNDInterpolator(tri, rav)
    interpolator_nearest = NearestNDInterpolator(tri, rav)
    dataint = interpolator(coords_coast)
    dataint[np.isnan(dataint)] = interpolator_nearest(coords_coast[np.isnan(dataint)])

    # if still nan values, it means it is ice - make it -9999
    dataint[np.isnan(dataint)] = -9999

    return dataint


def getLonLatData(data, sel, lon_xr="", lat_xr=""):
    datadim = data.dims

    if 'lat' in datadim:
        lon = data['lon']
        lat = data['lat']
        datawl = sel(lon=lon_xr, lat=lat_xr, method='nearest')

    elif 'latitude' in datadim:
        lon = data['longitude']
        lat = data['latitude']
        datawl = sel(longitude=lon_xr, latitude=lat_xr, method='nearest')

    elif 'x' in datadim:
        lon = data['x']
        lat = data['y']
        datawl = sel(x=lon_xr, y=lat_xr, method='nearest')

    if len(lon.shape) <= 1:
        lon, lat = np.meshgrid(lon, lat)

    return lon, lat, datawl

def getLonLatVars(data, sel):
    datadim = data.variables

    if 'lat' in datadim:
        lon = data['lon']
        lat = data['lat']

    elif 'latitude' in datadim:
        lon = data['longitude']
        lat = data['latitude']

    elif 'x' in datadim:
        lon = data['x']
        lat = data['y']

    if len(lon.shape) <= 1:
        lon, lat = np.meshgrid(lon, lat)

    return lon, lat


def coast_water_level_extract_multiple(datadir, outdir, maskdir, region, label, fstart, fend, addwave, dsettype='fcst'):

    if dsettype == 'fcst':
        dsethyd = dsets.datasets_hydro
        dsetwav = dsets.datasets_wave
    elif dsettype == 'reanal':
        dsethyd = dsets.datasets_hydro_reanal
        dsetwav = dsets.datasets_wave_reanal
    PHYvar = dsethyd[region]['var'][0]
    WAVvar = 'VHM0'

    fperiods = [fstart + datetime.timedelta(days=float(ii)) for ii in np.arange((fend - fstart).total_seconds() / 86400)]
    fperiode = [fperiodsi + datetime.timedelta(days=1) for fperiodsi in fperiods]

    saveConcatenatedNC(datadir, PHYvar, label, fperiods, fperiode, PHYvar + ".nc")
    if addwave:
        saveConcatenatedNC(datadir, "WAVsubset", label, fperiods, fperiode, "wav.nc")

    if not os.path.exists(outdir):
        os.mkdir(outdir)

    filepoints = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'output_points/output_locs_%s.txt' % region)
    data = pd.read_csv(filepoints, sep='\t')
    lond = data.iloc[:, 0].values
    latd = data.iloc[:, 1].values
    names_coast = data.iloc[:, 2].values
    coords_coast = np.vstack((lond, latd))

    # PHY(alternative) colocate using coordinates. what is dangerous is that if grid changes slightly, then we might hit a land (masked) point
    lon_xr = xr.DataArray(lond, dims='stations')
    lat_xr = xr.DataArray(latd, dims='stations')

    ncdata = xr.open_dataset(datadir + '/data/' + PHYvar + ".nc")  # concatenated in time
    LONwl, LATwl = getLonLatVars(ncdata, ncdata.sel)
    datadim = ncdata.dims

    # check is any items fell on masked land
    # if (('rlon', 'x') in datadim) or datawl[PHYvar].isel(time=0).any():
        # use kdtree instead of the non-nan model points
    maskarr = np.ravel(ncdata[PHYvar].isel(time=0).isnull())

    # LONw and LATwl alwys in lo,lat coordinates
    lonarr = np.ravel(LONwl)
    latarr = np.ravel(LATwl)

    # either this, or I need to extrapolate the data first...
    coordswl = np.column_stack((lonarr[~maskarr], latarr[~maskarr]))
    kdwl = KDTree(coordswl)
    kdist, indxwl = kdwl.query(coords_coast.T)

    lon_xr = xr.DataArray(coordswl[indxwl, 0], dims='stations')
    lat_xr = xr.DataArray(coordswl[indxwl, 1], dims='stations')

    if 'lat' in datadim:
        datawl = ncdata.sel(lon=lon_xr, lat=lat_xr, method='nearest')
    elif 'latitude' in datadim:
        datawl = ncdata.sel(longitude=lon_xr, latitude=lat_xr, method='nearest')
    elif 'x' in datadim:
        # find equivalent rotated coords
        kdreg = KDTree(np.column_stack((lonarr, latarr)))
        _, indxr = kdreg.query(np.column_stack((lon_xr, lat_xr)))

        indxr_mat = np.unravel_index(indxr, (LONwl.shape))
        lati_xr_r = xr.DataArray(indxr_mat[0], dims='stations')
        loni_xr_r = xr.DataArray(indxr_mat[1], dims='stations')

        datawl = ncdata.isel(x=loni_xr_r, y=lati_xr_r)

    # make those too far away nan
    #toofar = np.arange(len(lond))[kdist > 20 / 110]
    #newarr = datawl[PHYvar].values
    #newarr[:, toofar] = np.nan * np.ones(newarr[:, toofar].shape)

    #datawl[PHYvar] = xr.DataArray(
    #        data=newarr.T,
    #        dims=datawl.dims,
    #        coords=datawl.coords,
    #        attrs=datawl[PHYvar].attrs,
    #        )
    #drop those too far away and make unique- can be changed in the future
    not_toofar = np.arange(len(lond))[kdist <= 20 / 110]
    # datawl=datawl.isel(stations=not_toofar)
    # coords_coast=coords_coast[:,not_toofar]

    datatotal = datawl.copy()
    encodingspec = dict()
    encodingspec[dsethyd[region]['var'][0]] = {
                            "dtype": 'int16',
                            "scale_factor": float(0.001),
                            "add_offset": float(0),
                            "_FillValue":nc.default_fillvals['i2'],
                            # 'chunksizes': []
                            }

    if addwave:
        log.info('Extract wave for each parameter in data set')
        wavevarlist = dsetwav[region]['var'][0]

        try:
            ncdata_wav = xr.open_dataset(datadir + '/data/wav.nc')  # concatenated in time
            _, index = np.unique(ncdata_wav['time'], return_index=True)
            ncdata_wav = ncdata_wav.isel(time=index)

            LON, LAT = getLonLatVars(ncdata_wav, ncdata.sel)

            # LON, LAT, _ = getLonLatData(ncdata_wav, lambda **kwargs: None)

            log.info('Interpolate wave params to phy locations')
            # first create the wave kdtree
            # get mask
            lonarr = np.ravel(LON)
            latarr = np.ravel(LAT)
            if 'summer_mask' in dsetwav[region].keys():
                # get mask from predefined field
                if isinstance(dsetwav[region]['summer_mask'], dict):
                    # check for which mask to use
                    for item in dsetwav[region]['summer_mask']:
                        if fstart >= datetime.datetime.strptime(item, '%Y%m%d'):
                            refdata = xr.open_dataset(os.path.join(maskdir, dsetwav[region]['summer_mask'][item]['filename']))
                            refdata = refdata[dsetwav[region]['summer_mask'][item]['var']]
                else:
                    refdata = xr.open_dataset(os.path.join(maskdir, dsetwav[region]['summer_mask']))
                    refdata = refdata['VHM0']

                plotfield = refdata.isel(time=0)  # example field
                tempmask = np.isnan(plotfield.values)  # #WARNING! Wave fields showing time-dependent mask fields (or at least, some invalid points sometimes)
                maskfieldW = tempmask.copy()

                vhmot0 = ncdata_wav['VHM0'].isel(time=0)
                if not tempmask.shape == vhmot0.shape:
                    # if just one row difference, gind nearest neighbor
                    if np.abs(tempmask.shape[0] - vhmot0.shape[0]) <= 1 and np.abs(tempmask.shape[1] - vhmot0.shape[1]) <= 1:
                        # just find nearest
                        [LONmsk, LATmsk] = np.meshgrid(refdata['lon'].values, refdata['lat'].values)
                        lonarrmask = np.ravel(LONmsk)
                        latarrmask = np.ravel(LATmsk)
                        kdmskorg = KDTree(np.vstack((lonarrmask, latarrmask)).T)
                        kdist, indxmsk = kdmskorg.query(np.vstack((lonarr, latarr)).T)
                        maskfieldW = np.ravel(tempmask)[indxmsk]
                        maskfieldW = maskfieldW.reshape(LON.shape)
                    else:
                        raise  RuntimeError('Mask does not fit wave grid. Stop\n')
            else:
                # get mask from this dataset
                plotfield = ncdata_wav['VHM0'].isel(time=0)  # example field
                maskfieldW = np.isnan(plotfield.values)  # #WARNING! Wave fields showing time-dependent mask fields (or at least, some invalid points sometimes)

            maskarr = np.ravel(maskfieldW)
            coords = np.vstack((lonarr[~maskarr], latarr[~maskarr]))  # either this, or I need to extrapolate the data first...

            tri = Delaunay(coords.T)  # Compute the triangulation, so we can reuse it
            dateswa = matplotlib.dates.date2num(ncdata_wav['time'].values)
            dateswl = matplotlib.dates.date2num(ncdata['time'].values)

            transp = coords_coast.T

            pool = mp.Pool()
            for varw in wavevarlist:
                dataWAVfield = np.ones((len(lond), ncdata_wav.dims['time'])) * np.nan

              # colocate in space
                vals = ncdata_wav[varw].values
                result = pool.map(partial(calc_interp, ncdataWAV=vals, maskarr=maskarr, coords_coast=transp, tri=tri), range(len(ncdata_wav['time'])))
                for idx, val in enumerate(result):
                    dataWAVfield[:, idx] = val

                # colocate in time.
                # could interpolate in time in one go by using interp2d (numpoints*time=field)
                f = interpolate.interp2d(dateswa, np.arange(coords_coast.shape[1]), dataWAVfield)
                # no idea what happens if the wave datasets spans less time (i.e. extrapolation). we could always take a day before and after for overlap
                data_wavfield_int = f(dateswl, np.arange(coords_coast.shape[1]))

                # remask
                data_wavfield_int[data_wavfield_int < 0] = np.nan

                data_wavfield_xr = xr.DataArray(
                    data=data_wavfield_int,
                    dims=datawl.dims,
                    coords=datawl.coords,
                    attrs=ncdata_wav[varw].attrs
                )

                datatotal = datatotal.assign({varw:data_wavfield_xr})
                if varw in ['VMDR', 'VPED']:
                    scf = float(0.1)
                else:
                    scf = float(0.01)
                encodingspec[varw] = { "dtype": 'int16',
                                    "scale_factor": scf,
                                    "add_offset": float(0),
                                    "_FillValue": nc.default_fillvals['i2'],
                                    # 'chunksizes': []
                                    }
        except Exception as e:
            log.error(e)
            log.info("Creating an empty dataset")
            for varw in wavevarlist:
                # create empty field, so that file size remains the same always
                data_wavfield_xr = xr.DataArray(
                    data=np.ones(datawl[PHYvar].shape) * np.nan,
                    dims=datawl.dims,
                    coords=datawl.coords
                )
                datatotal = datatotal.assign({varw:data_wavfield_xr})
                scf = float(0.01)
                encodingspec[varw] = { "dtype": 'int16',
                                    "scale_factor": scf,
                                    "add_offset":float(0),
                                    "_FillValue":nc.default_fillvals['i2'],
                                    # 'chunksizes': []
                                    }

    obsnames = xr.DataArray(
        data=np.array(names_coast, dtype='S50'),
        dims=datawl.stations.dims,
        coords=datawl.stations.coords,
        attrs={'standard_name': 'station_name',
                'long_name': 'station_name'}
    )

    datatotal = datatotal.assign(stnames=obsnames)
    datatotal[PHYvar]=datatotal[PHYvar].transpose("stations", "time")
    datatotal=datatotal.isel(stations=not_toofar)
    # rename coordinates to longitude and latitude
    # if 'lon' in datatotal.variables.keys():
    #     datatotal = datatotal.rename({'lon':'longitude', 'lat':'latitude'})

    fileout = 'tseries_coastal_%s_%s_%s.nc' % (label, fperiods[0].strftime('%Y-%m-%d_%H-%M-%S'), fperiode[-1].strftime('%Y-%m-%d_%H-%M-%S'))
    datatotal.encoding['zlib'] = True  # Conserved
    datatotal.to_netcdf(os.path.join(outdir, fileout), encoding=encodingspec)
    # // TODO: Check result of check and possibly stop?
    utils.check_timeseries_output_size(os.path.join(outdir, fileout), dsethyd[region]['tssize'], 0.2)

    log.info('=====Time series for %s: %s /%s generated\n=====' % (region, fperiods[0].strftime('%Y-%m-%d_%H-%M-%S'), fperiode[-1].strftime('%Y-%m-%d_%H-%M-%S')))

    # else:
    #     log.info('=====Files for %s: %s /%s not there yet, will try again next day\n=====' % (region, fperiods[0].strftime('%Y-%m-%d_%H-%M-%S'), fperiode[-1].strftime('%Y-%m-%d_%H-%M-%S')))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Feb 19 16:45:04 2021

@author: mirazoki
test to download NRT Insitu obs
"""
import datetime
import logging
import os
import urllib.request

import numpy as np
import pandas as pd

log = logging.getLogger(__name__)


def download_insitu_obs(region, dateini, dateend, obstype='latest', chosen=['TG'], outdir='.'):

	datasets = {'TG':['SLEV'],
						'MO':['SWHT', 'VSPEC1D'],
						'ARC':{'strname':'arc', 'folder':'INSITU_ARC_NRT_OBSERVATIONS_013_031'},
						# 'ARC_tsm':{'strname':'arc','folder':'INSITU_ARC_NRT_OBSERVATIONS_013_031'},
						'BAL':{'strname':'bal', 'folder':'INSITU_BAL_NRT_OBSERVATIONS_013_032'},
						'BS':{'strname':'bs', 'folder':'INSITU_BS_NRT_OBSERVATIONS_013_034'},
						'MED':{'strname':'med', 'folder':'INSITU_MED_NRT_OBSERVATIONS_013_035'},
						'IBI':{'strname':'ibi', 'folder':'INSITU_IBI_NRT_OBSERVATIONS_013_033'},
						# 'IBI_15min':{'strname':'ibi','folder':'INSITU_IBI_NRT_OBSERVATIONS_013_033'},
						'NWS':{'strname':'nws', 'folder':'INSITU_NWS_NRT_OBSERVATIONS_013_036'}
						# 'NWS_15min':{'strname':'nws','folder':'INSITU_NWS_NRT_OBSERVATIONS_013_036'}
						}
	if region in datasets.keys():
			if not os.path.exists(outdir):
					os.mkdir(outdir)

			# outdirdata=os.path.join(outdir,obstype)
			# if not os.path.exists(outdirdata):
			# 		  os.mkdir(outdirdata)
			colidx = ['product_id', 'file_name', 'geospatial_lat_min', 'geospatial_lat_max', 'geospatial_lon_min', 'geospatial_lon_max', 'time_coverage_start', 'time_coverage_end', 'institution', 'date_update', 'data_mode', 'parameters']
			# obs_sets=['history']#,'monthly','latest']
			# chosen=['MO','TG']

			indexfile = r'/Core/%s/%s_multiparameter_nrt/index_%s.txt' % (datasets[region]['folder'], datasets[region]['strname'], obstype)
			# ftp_add=r'/Core/%s/%s_multiparameter_nrt/%s' %(datasets[region],region.lower(),obstype)
			# dateini=datetime.datetime(2021,2,22)
			# dateend=datetime.datetime(2021,2,21)
			# urllib.urlretrieve('ftp://username:password@server/path/to/file', 'file')

			# download index file
			urllib.request.urlretrieve('ftp://mirazoquiapecec:MaialenCMEMS2017@nrt.cmems-du.eu%s' % indexfile, os.path.join(outdir, 'index_%s.txt' % obstype))

			id_data = pd.read_csv(os.path.join(outdir, 'index_%s.txt' % obstype), sep=',', delimiter=None, header=None, skiprows=6, names=colidx, index_col=None)

			for obsi in chosen:
					searchvars = datasets[obsi]
					boolvar = [np.any([vari in row['parameters'] for vari in searchvars]) for ii, row in id_data.iterrows()]

					if obstype == 'latest':
							# filter to the dates we need
							datestrlist = [datetime.datetime.strftime(dateini + datetime.timedelta(days=i), "%Y%m%d") for i in np.arange((dateend - dateini).total_seconds() / 86400.0)]
							# datestr=dateini.strftime("%Y%m%d")
							# filter those for the chosen times
							boolvar_un = [np.any([datestr in row['file_name'] for datestr in datestrlist]) for ii, row in id_data.iterrows()]

							boolvar = np.array(boolvar) & np.array(boolvar_un)

							# listncs=[pathlib.Path(filei).parts[-1].replace('.nc','') for filei in id_data['file_name'].loc[boolvar]]
							listncfull = id_data['file_name'].loc[boolvar]
							# listobs=['_'.join(nci.split('_')[0:-1]) for nci in listncs]

					# download list
					for ii, inc in enumerate(listncfull):
							outfile = os.path.join(outdir, inc[inc.find(obstype):])
							outdiri = os.path.split(outfile)[0]

							if not os.path.exists(os.path.split(outdiri)[0]):  # parent directory
									os.mkdir(os.path.split(outdiri)[0])
							if not os.path.exists(outdiri):
									os.mkdir(outdiri)

							address = inc.replace('ftp://', '')
							if not os.path.isfile(outfile):
									try:
											urllib.request.urlretrieve('ftp://mirazoquiapecec:MaialenCMEMS2017@%s' % address, filename=outfile)
									except:
											log.error("Failed to retrieve observation %s: %s - %s\n" % (inc, dateini.strftime("%Y%m%d"), dateend.strftime("%Y%m%d")))



#!/usr/bin/env python
# coding: utf-8
'''
This module contains calls to the FES2014 tide model
'''

import os
import numpy as np
import datetime
import pyfes as fes
import argparse

from numpy import float64

import logging
log = logging.getLogger(__name__)


#------------------------------------------------------------------------------
def init_fes_h(fesdir, prodname='org'):
    file_FES_conf_ocean = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'ocean_tide.ini')
    file_FES_conf_load = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'load_tide.ini')
    file_FES_conf_ocean_extrap = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'ocean_tide_extrapolated.ini')

    if prodname == 'org':
        short_tide = fes.Handler("ocean", "memory", file_FES_conf_ocean)
    elif prodname == 'extrap':
        short_tide = fes.Handler("ocean", "memory", file_FES_conf_ocean_extrap)
    radial_tide = fes.Handler("radial", "memory", file_FES_conf_load)
    log.info('* init FES2014 tide model')
    return short_tide, radial_tide

#------------------------------------------------------------------------------
# def compute_uv_tide_FES(date, lon, lat, eastward_velocity, northward_velocity):

#     log.info('- compute tide currents for '+str(date))
#     lon=float64(lon); lat=float64(lat)
#     lons, lats = np.meshgrid(lon, lat)
#     npj,npi=lons.shape
#     dates = np.empty(lons.shape, dtype='datetime64[us]')
#     dates.fill(date)

#     u, lp   = eastward_velocity.vector(lats.ravel(), lons.ravel(), dates.ravel())
#     v, _    = northward_velocity.vector(lats.ravel(), lons.ravel(), dates.ravel())

#     # convert result in m/s
#     u=u*0.01 ; v=v*0.01
#     u = u.reshape((npj, npi))
#     v = v.reshape((npj, npi))
#     return u, v

# #-----------------------------------------------------------------------------
# def compute_uv_tide_FES(date, lon, lat, eastward_velocity, northward_velocity):
#     # test version to save time
#         log.info('- compute tide currents for '+str(date))
#         lon=float64(lon); lat=float64(lat)
#         lons, lats = np.meshgrid(lon, lat)
#         npj,npi=lons.shape
#         dates = np.empty(lons.shape, dtype='datetime64[us]')
#         dates.fill(date)

#         u, lp   = eastward_velocity.vector(lats.ravel(), lons.ravel(), dates.ravel())
#         v, _    = northward_velocity.vector(lats.ravel(), lons.ravel(), dates.ravel())

#         # convert result in m/s
#         u=u*0.01 ; v=v*0.01
#         u = u.reshape((npj, npi))
#         v = v.reshape((npj, npi))
#         return u, v


#------------------------------------------------------------------------------
def compute_geo_tide_FES(date, lon, lat, short_tide, radial_tide):  # compute FES tide for a grid. we normally have more locations than times, this could be smart to do...

    log.info('- compute tide elevation for ' + str(date))
    lon = float64(lon); lat = float64(lat)
    lons, lats = np.meshgrid(lon, lat)
    npj, npi = lons.shape
    dates = np.empty(lons.shape, dtype='datetime64[us]')
    dates.fill(date)

    tide, lp = short_tide.vector(lats.ravel(), lons.ravel(), dates.ravel())
    load, _ = radial_tide.vector(lats.ravel(), lons.ravel(), dates.ravel())

    # convert result in meters
    geo_tide = (tide + lp + load) * 0.01
    geo_tide = geo_tide.reshape((npj, npi))
    return geo_tide


def compute_geo_tide_FES_array(dates, lon, lat, short_tide, radial_tide):  # compute FES tide for one position, for a number of times
    lons = np.empty(len(dates), dtype=float64)
    lats = np.empty(len(dates), dtype=float64)
    lons.fill(lon)
    lats.fill(lat)
    tide, lp, _ = short_tide.calculate(lons, lats, dates.astype(dtype='datetime64[us]'))
    load, _, _ = radial_tide.calculate(lons, lats, dates.astype(dtype='datetime64[us]'))

    # convert result in meters
    geo_tide = (tide + lp + load) * 0.01

    # geo_tide = np.ma.masked_where(np.isnan(geo_tide), geo_tide)

    return geo_tide


import os
import pytest
import argparse
import ecfas.cmems_operational_fullworkflow as wf


def setup_module():
    global test_dir
    test_dir = os.path.dirname(os.path.abspath(__file__))


def test_workflow_no_config_file():
    """ Test behavior when no config file is specified """
    cli_args = argparse.Namespace()
    cli_args.region = 'NWS'
    cli_args.reanal = False
    cli_args.t0 = None
    cli_args.debug = False
    with pytest.raises(SystemExit) as pytest_wrapped_e:
        wf.execute_workflow(cli_args)
    assert pytest_wrapped_e.type == SystemExit
    assert pytest_wrapped_e.value.code == 1



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Mar 15 16:21:43 2021

@author: mirazoki
"""
from _ast import Try
import datetime
import glob
import itertools
import logging
import os
import sys

import ecfas.cmems_datasets as dsets
import ecfas.fes.module_FES2014 as fes
import ecfas.swash_formulations as sf
import ecfas.utils as utils
import netCDF4 as nc
import numpy as np
import xarray as xr
import pandas as pd
from scipy.spatial import cKDTree as KDTree

log = logging.getLogger(__name__)


def split_by_chunks(dataset):
    chunk_slices = {}
    for dim, chunks in dataset.chunks.items():
        slices = []
        start = 0
        for chunk in chunks:
            if start >= dataset.sizes[dim]:
                break
            stop = start + chunk
            slices.append(slice(start, stop))
            start = stop
        chunk_slices[dim] = slices
    for slices in itertools.product(*chunk_slices.values()):
        selection = dict(zip(chunk_slices.keys(), slices))
        yield dataset[selection]
        
def get_MDT(region,maskdir,coords):
    if 'ARC' in region:
        data=xr.open_mfdataset(os.path.join(maskdir,'MDT_CNES_CLS18.nc'))
        longitude=data['longitude'].values
        mdtobsval=data['mdt'].isel(time=0).values
        
        left=longitude>180
        right=longitude<=180
        lonnew=np.hstack((longitude[left]-360,longitude[right]))
        # Xobs,Yobs=np.meshgrid(lonnew,data['latitude'])        
        mdtobsnew=np.hstack((mdtobsval[:,left],mdtobsval[:,right]))
        data=xr.Dataset(
            data_vars={'mdt':(['latitude','longitude'], mdtobsnew) ,
                    },
                    coords=dict(
                        longitude=lonnew,
                        latitude=data['latitude'].values,
                    ),
                    attrs=data.attrs,
                )        
    else:
        data=xr.open_mfdataset(os.path.join(maskdir,'%s_mdt.nc' %region))
    plotfield=data['mdt']#example field
    maskfield=np.isnan(plotfield.values)
    maskarr=np.ravel(maskfield)
    [LON,LAT]=np.meshgrid(data['longitude'],data['latitude'])
    lonarr=np.ravel(LON) #LONw and LATwl alwys in lo,lat coordinates
    latarr=np.ravel(LAT)
    # mdtarr=np.ravel(plotfield)[~maskarr]
    coordsmdt=np.column_stack((lonarr[~maskarr],latarr[~maskarr])) #either this, or I need to extrapolate the data first...
    kdwl=KDTree(coordsmdt)
    kdist, indxwl = kdwl.query(coords)
    # outmdt=mdtarr[indxwl]
    lon_xr = xr.DataArray(coordsmdt[indxwl,0], dims='stations')
    lat_xr = xr.DataArray(coordsmdt[indxwl,1], dims='stations')   

    outdata=plotfield.sel(longitude= lon_xr, latitude = lat_xr,method='nearest')
    
    return outdata


def coast_water_level_process(region, tsdir, outdir, label,maskdir, swashtype='cte', betatype='None', fes_tide=None):

    filename = glob.glob(os.path.join(tsdir, 'tseries_coastal_%s_*.nc' % (label)))[0]

    if len(filename) == 0:
        log.error('Error. Timeseries file %s is empty. Stopping analysis' % filename)
        os.system.exit(1)

    datamodel = xr.open_dataset(filename)
    if 'longitude' in datamodel.variables.keys():
             datamodel = datamodel.rename({'longitude':'lon', 'latitude':'lat'})
    for coordi in datamodel.coords:
        if coordi not in ['lon','lat','stations','time']:
            datamodel=datamodel.drop_vars([coordi])

    phy_var = dsets.datasets_hydro[region]['var'][0]
    # change PHYvar to standard 'zos' for all domains
    if phy_var in datamodel.variables.keys():
        datamodel = datamodel.rename({phy_var:'zos'})
    # make stnames the stations coordinate
    datamodel = datamodel.assign_coords(stations=datamodel['stnames'].values)

    encodingspec = dict()
    # check if other elements need loading up
    if fes_tide is not None:
        # include FES tide at closest point
        # build kdtree
        examplegrid = os.path.join(fes_tide['fesdir'], 'ocean_tide/m2.nc')
        fessample = xr.open_dataset(examplegrid)
        # plt.pcolormesh(fessample['lon'],fessample['lat'],fessample['amplitude'])
        Xfes, Yfes = np.meshgrid(fessample['lon'], fessample['lat'])
        # plt.pcolormesh(Xfes,Yfes,fessample['amplitude'])
        maskfes = np.isnan(fessample['amplitude'])
        xfesarr = Xfes[~maskfes].flatten()
        yfesarr = Yfes[~maskfes].flatten()
        coordsfes = np.column_stack((xfesarr, yfesarr))  # either this, or I need to extrapolate the data first...
        kdfes = KDTree(coordsfes)

        lonr = datamodel['lon'].values.copy()
        lonr[lonr < 0] = lonr[lonr < 0] + 360
        latr = datamodel['lat'].values.copy()

        kdist, indxfes = kdfes.query(np.hstack((np.reshape(lonr, (lonr.size, 1)), np.reshape(latr, (latr.size, 1)))))

        # initialize tide
        tideseries = np.ones(datamodel['zos'].values.shape) * np.nan
        for ipoint, ix in enumerate(indxfes):
            tideseries[ipoint,:] = fes.compute_geo_tide_FES_array(datamodel['time'].values, xfesarr[ix], yfesarr[ix], fes_tide['short_tide'], fes_tide['radial_tide'])

        tideseries_xr = xr.DataArray(
            data=tideseries,  # initialize to zero
            dims=datamodel.dims,
            coords=datamodel.coords,
            attrs={'standard_name': 'tide_FES',
                'long_name': 'tide retrieved from FES 2014 model',
                'units':'m'})

        xrtwl = datamodel['zos'] + tideseries_xr
        datamodel['zos'] = xrtwl

    if region == 'ARC':
        fileocean = filename.replace(region, 'ARC_ocean')
        if not os.path.exists(fileocean):
            log.error('Missing OCEAN file for t0 %s' % label)
            sys.exit(1)
        else:
            datamodel_ocean = xr.open_dataset(fileocean)
            ovar = dsets.datasets_hydro['ARC_ocean']['var'][0]
            # rename things
            if 'longitude' in datamodel_ocean.variables.keys():
                datamodel_ocean = datamodel_ocean.rename({'longitude':'lon', 'latitude':'lat'})
            for coordi in datamodel_ocean.coords:
                if coordi not in ['lon','lat','stations','time']:
                    datamodel_ocean=datamodel_ocean.drop_vars([coordi])
            if ovar in datamodel_ocean.variables.keys():
                datamodel_ocean = datamodel_ocean.rename({ovar:'zos'})

            coordswl = np.column_stack((datamodel['lon'].values, datamodel['lat'].values))  # either this, or I need to extrapolate the data first...
            coordswl_ocean = np.column_stack((datamodel_ocean['lon'].values, datamodel_ocean['lat'].values))  # either this, or I need to extrapolate the data first...

            kdwlocean = KDTree(coordswl_ocean)
            kdist, indxwl = kdwlocean.query(coordswl)

            toceaninterp = datamodel_ocean['zos'].isel(stations=indxwl)
            toceaninterp=toceaninterp.assign_coords(stations=datamodel['stnames'].values)
            _, index = np.unique(toceaninterp['time'], return_index=True)
            toceaninterp = toceaninterp.isel(time=index)
            _, index = np.unique(datamodel['time'], return_index=True)
            datamodel = datamodel.isel(time=index)

            xrtwl = datamodel['zos'] + toceaninterp.interp(time=datamodel['time'])  # maybe I need to interpolate to common times?
            datamodel['zos'] = xrtwl

    #make unique
    coords_coast=np.vstack((datamodel['lon'].values,datamodel['lat'].values)).T
    unlist,unidx=np.unique(coords_coast,axis=0,return_index=True)
    unidxs=np.sort(unidx)
    datamodel=datamodel.isel(stations=unidxs)
    
    #add MDT
    coords=np.vstack((datamodel['lon'].values,datamodel['lat'].values)).T
    mdt=get_MDT(region,maskdir,coords)
    if 'longitude' in mdt.coords:
        mdt=mdt.rename({'longitude':'lon', 'latitude':'lat'})
    mdt=mdt.assign_coords(lon=datamodel['lon'],lat=datamodel['lat'])
    datamodel['zos']=datamodel['zos']-mdt
    
    # add wave contributions
    # initialize
    datasetup_xr = xr.DataArray(
        data=np.zeros(datamodel['zos'].shape),  # initialize to zero
        dims=datamodel.dims,
        coords=datamodel.coords,
        attrs={'standard_name': 'wave_setup',
                'units':'m'}
        )

    datamodel = datamodel.assign(Vsetup=datasetup_xr.transpose("stations", "time"))
    if swashtype == 'cte':
        eta, _, _, R, irb = sf.Hsprop(datamodel['VHM0'].values)
        betatype = 'dissipative'
    elif swashtype == 'stockdon':
        if betatype is None:
            beta = np.zeros(datamodel[phy_var].shape)  # force the dissipative formulation of stockdon
            betatype = 'dissipative'
        elif betatype == 'Athanasios':
            beta = sf.slopes_athan(datamodel['lon'].values, datamodel['lat'].values)
        beta[np.isnan(beta)] = 0.0  # force it to be dissipative,beta independent
        eta, _, _, R, irb = sf.stockdon(datamodel['VHM0'].values,
                                           datamodel['VTPK'].values, beta=beta)

    datamodel['Vsetup'].values = eta
    datamodel['Vsetup'].attrs['long_name'] = 'wave setup calculated using %s parameterization and %s slope' % (swashtype, betatype)

    # add enconding
    encodingspec["Vsetup"] = {"dtype": 'int16',
                "scale_factor": float(0.001),
                "add_offset": float(0.0),
                "_FillValue":nc.default_fillvals['i2'],
                # 'chunksizes': []
                }
    datamodel.encoding['zlib'] = True  # Conserved

    datamodel.attrs['easting'] = 'lon'
    datamodel.attrs['northing'] = 'lat'

    if not os.path.exists(outdir):
        os.mkdir(outdir)
    # write out independent days
    dayarray = pd.date_range(start=datamodel.indexes['time'][0], end=datamodel.indexes['time'][-1], freq='1D', closed='left')
    bday = datetime.datetime.strptime(label, '%Y-%m-%d_%H-%M-%S').strftime('%Y%m%d%H%M%S')
    for iday in dayarray:
        tini = iday.to_pydatetime()
        tend = (iday.to_pydatetime() + datetime.timedelta(hours=23))
        dataday = datamodel.sel(time=slice(tini, tend))
        if dataday.dims['time'] > 0:
            fileout = 'TScoast_%s_b%s_%s_%s.nc' % (region, bday, tini.strftime('%Y%m%d'), (tend + datetime.timedelta(hours=1)).strftime('%Y%m%d'))
            dataday.to_netcdf(os.path.join(outdir, fileout), encoding=encodingspec)

    # // TODO: Check result of check and possibly stop?
    utils.check_timeseries_output_size(os.path.join(outdir, fileout), dsets.datasets_hydro[region]['tsdsize'], 0.25)

    log.info('=====Time series for %s: label %s  processed\n=====' % (region, label))

    return os.path.join(outdir, fileout)


import os
import ecfas.cmems_datasets as ds
import ecfas.motu_download as md
import ecfas.utils as utils
import pytest


def setup_module():
    global out_dir
    out_dir = 'test_outputs'
    utils.clean_dir(out_dir)
    global motu
    motu = 'http://nrt.cmems-du.eu/motu-web/Motu'
    global user
    user = os.environ['CMEMS_USR']
    global password
    password = os.environ['CMEMS_PWD']


def test_fail_download_wrong_url():
    t_start = '2021-05-01 00:00:00'
    t_end = '2021-05-02 00:00:00'
    with pytest.raises(SystemExit) as pytest_wrapped_e:
        md.motu_download(user, password, 'http://nrt.cmems-du.com/motu-web/Motu', t_start, t_end, out_dir, ds.datasets_hydro, "PHY", region='NWS')
    assert pytest_wrapped_e.type == SystemExit
    assert pytest_wrapped_e.value.code == 1


def test_download_hydroset():
    t_start = '2021-05-01 00:00:00'
    t_end = '2021-05-02 00:00:00'
    md.motu_download(user, password, motu, t_start, t_end, out_dir, ds.datasets_hydro, "PHY", region='NWS')


def test_download_waveset():
    t_start = '2021-05-01 00:00:00'
    t_end = '2021-05-02 00:00:00'
    md.motu_download(user, password, motu, t_start, t_end, out_dir, ds.datasets_wave, "WAV", region='NWS')


def test_download_satset():
    t_start = '2021-05-01 00:00:00'
    t_end = '2021-05-02 00:00:00'
    md.motu_download_satdata(user, password, motu, t_start, t_end, out_dir)


#!/usr/bin/env python3
"""
Created on Thu Feb  4 14:53:13 2021

@author: mirazoki

wave setup and runup parameterizations
"""
import logging

import cartopy
from cartopy.mpl.gridliner import LONGITUDE_FORMATTER, LATITUDE_FORMATTER
from matplotlib.gridspec import GridSpec
from mpl_toolkits.axes_grid1 import make_axes_locatable
from scipy.spatial import cKDTree as KDTree

import cartopy.crs as ccrs
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd

log = logging.getLogger(__name__)


def shoal_K(T0, h):
    L0 = (9.81 * T0 ** 2) / 2 / np.pi
    a = h / L0
    # obtain the deep water equivalent wave height through linear wave theory.
    Ks = 1 / np.sqrt(np.tanh(2 * np.pi * a) * (1 + 4 * np.pi * a / np.sinh(4 * np.pi * a)))
    return Ks


def shoal_K_GL(a):
    # obtain the deep water equivalent wave height through linear wave theory.Green's law for shallow waters
    Ks = 1 / np.sqrt(np.sqrt(a * 2 * np.pi) * (1 + 4 * np.pi * a / np.sinh(4 * np.pi * a)))
    return Ks


def stockdon(H0, T0, beta=0.01):  # most common slope for sandy beaches, comparable to EU median in Athanasios
    L0 = (9.81 * T0 ** 2) / 2 / np.pi
    wsteep = H0 / L0
    irb = beta / (wsteep ** 0.5)
    diss = irb < 0.3
    # initialize
    eta = np.zeros(H0.shape)
    S_ig = np.zeros(H0.shape)
    S_inc = np.zeros(H0.shape)
    R2 = np.zeros(H0.shape)

    # general formulation
    eta = 0.35 * beta * (H0 * L0) ** 0.5
    S_ig = 0.06 * (H0 * L0) ** 0.5
    S_inc = 0.75 * beta * (H0 * L0) ** 0.5

    # substitue where dissipative
    eta[diss] = 0.016 * (H0[diss] * L0[diss]) ** 0.5
    S_ig[diss] = 0.046 * (H0[diss] * L0[diss]) ** 0.5
    S_inc[diss] = 0.0

    # runup
    R2 = 1.1 * (eta + 0.5 * (S_ig + S_inc))
    return eta, S_ig, S_inc, R2, irb


def Hsprop(H0, cte=0.2):
    eta = cte * H0
    S_ig = np.zeros(H0.shape)
    S_inc = np.zeros(H0.shape)
    R2 = np.zeros(H0.shape)
    irb = np.zeros(H0.shape)

    return eta, S_ig, S_inc, R2, irb


def slopes_athan(lonarray, latarray):
    # read slopes from Athanasios dataset
    filename = r'/homelocal2/mirazoki/ECFAS/data/Slopes/nearshore_slopes_lo.csv'
    dataslopes = pd.read_csv(filename, sep=',', delimiter=None, index_col=None)

    # find the closest non-nan given certain distance. also discard other dodgy values (only pick 0 and 6)
#   			0: No errors or warnings
# 				1: Error- Number of cross-shore underwater points not enough for analysis
# 				2: Warning - DoC deeper that most deep offshore point (used that one as DoC)
# 				3: Error - Shoreline point not found
# 				4: Error - DoC point not found
# 				5: Warning - Negative calculated  slope
# 				6: Warning - Really steep slope (step) close to MSL
# 				7: Error - No DoC estimation available
# 				8: Warning 1 and 6

    dataslopes = dataslopes.loc[dataslopes['error_code'].isin([0, 6])]
    coords = np.column_stack((dataslopes['X'].values, dataslopes['Y'].values))  # either this, or I need to extrapolate the data first...
    kdslope = KDTree(coords)
    coordsreq = np.hstack((lonarray.reshape(lonarray.size, 1), (latarray.reshape(latarray.size, 1))))
    dist, i = kdslope(coordsreq)
    reqslopes = dataslopes['slope'].iloc[i]
    return reqslopes

    # #plot to check
    # bbox=[-32,35,25,80]
    # figid=plt.figure(figsize=(12,12))
    # axloc = plt.gca(projection=ccrs.PlateCarree())
    # axloc.add_feature(cartopy.feature.LAND.with_scale('10m'),facecolor='gray',zorder=-1,alpha=1, edgecolor='black')
    # axloc.set_extent(bbox)
    # im=axloc.scatter(dataslopes['X'],dataslopes['Y'],10,dataslopes['slope'],cmap='jet',vmin=0,vmax=1,transform=ccrs.PlateCarree())
    # im2=axloc.scatter(lonarray,latarray,10,transform=ccrs.PlateCarree())

    # figid.colorbar(im,orientation='vertical')

    # bbox=[-32,35,25,80]
    # figid=plt.figure(figsize=(12,12))
    # axloc = plt.gca(projection=ccrs.PlateCarree())
    # axloc.add_feature(cartopy.feature.LAND.with_scale('10m'),facecolor='gray',zorder=-1,alpha=1, edgecolor='black')
    # axloc.set_extent(bbox)
    # im=axloc.scatter(lonarray,latarray,10,transform=ccrs.PlateCarree())
    # figid.colorbar(im,orientation='vertical')

    # request model locations
    kdist, indx = kdslope.query(np.vstack((lonarray, latarray)).T)
    kdist, indx = kdslope.query(np.vstack((lonarray[0], latarray[0])).T)

    beta = dataslopes['slope'].iloc[indx].values

    beta[kdist >= 0.1] = np.nan

    return beta

# T0=10
# h=np.arange(1,300)

# L0=(9.81*T0**2)/2/np.pi
# a=h/L0

# Kslist=np.array([shoal_K(ai) for ai in a])
# Kslist_GL=np.array([shoal_K_GL(ai) for ai in a])

# fig=plt.figure(1)
# plt.plot(a,Kslist,'-r')
# ax.plot(a,Kslist_GL,'-g')
# ax=plt.gca()
# ax.semilogx()

# fig=plt.figure(2)
# plt.plot(a,Kslist_GL)
# ax=plt.gca()
# ax.semilogx()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Mar 18 17:20:10 2021

@author: mirazoki
"""

import argparse
import glob
import os
import sys
import logging

import datetime as dt
import ecfas.cmems_datasets as dsets
import ecfas.coast_water_level_validate as cv
import numpy as np

log = logging.getLogger(__name__)
def validate(outdir, region, t_start, t_end):

    if '15min' in region:
        regobs = region.replace('_15min', '')
    elif 'tsm' in region:
        regobs = region.replace('_tsm', '')
    else:
        regobs = region
    obsdir = os.path.join(outdir, regobs, 'observations')

    if t_start == 'None' and t_end == 'None':
        # validate the t0s of the last 5 days
        today = dt.datetime(dt.datetime.now().year, dt.datetime.now().month, dt.datetime.now().day)
        trangeval_start = today - dt.timedelta(days=5)
        trangeval_stop = trangeval_start + dt.timedelta(days=1)

    else:
        if t_start != 'None':
            # prescribing also the t0 list
            t0 = dt.datetime.strptime(t_start, "%Y-%m-%d %H-%M-%S")  # YYYYMMDD
            trangeval_start = t0
            if t_end == 'None':
                trangeval_stop = t0 + dt.timedelta(days=5)
            else:
                # prescribing also the t0 range
                trangeval_stop = dt.datetime.strptime(t_end, "%Y-%m-%d %H-%M-%S")  # YYYYMMDD

    trangeval = [trangeval_start, trangeval_stop]  # list of t0s

    obstype = 'latest'  # latest,monthly,history
    phy_var = dsets.datasets_hydro[region]['var'][0]

    listzos = glob.glob(os.path.join(outdir, region, 'timeseries', 'processed', 'tseries_coastal_*.nc'))

    listzos.sort()
    t0list = []
    for filei in listzos:
        parts = os.path.split(filei)[-1].split('_')
        t0list.append(dt.datetime.strptime(parts[-6] + '_' + parts[-5], "%Y-%m-%d_%H-%M-%S"))

    t0list, idxu = np.unique(t0list, return_index=True)
    listzos = np.array(listzos)[idxu]

    if len(trangeval) > 0:
        # clip the t0s to the given range
        boolsel = [t0i >= trangeval[0] and t0i < trangeval[1] for t0i in t0list]
    else:
        boolsel = np.ones(listzos.shape, dtype='bool')
    tsel = t0list[np.array(boolsel)]
    filesel = listzos[np.array(boolsel)]

    # TODO: how does this fail?
    for ifile, filename in enumerate(filesel):
        parts = os.path.split(filename)[-1].split('_')
        tini = dt.datetime.strptime(parts[-4] + '_' + parts[-3], "%Y-%m-%d_%H-%M-%S")
        tend = dt.datetime.strptime(parts[-2] + '_' + parts[-1].replace('.nc', ''), "%Y-%m-%d_%H-%M-%S")
        outdir = os.path.join(os.path.split(filename)[0], 'figures')
        cv.coast_water_level_validate(region, phy_var, outdir, filename, tini, tend, obstype, t0=tsel[ifile], obsdir=obsdir)

    return True


def main():
    parser = argparse.ArgumentParser("op_validate")
    parser.add_argument('-o', '--outputs', metavar='<output_dir>', required=True, help='Absolute path to output data to be checked')
    parser.add_argument('-r', '--region', metavar='<region>', required=True, help='Region of interest, one of ARC, BAL, BS. IBI, MED, NWS, GLO. Defaults to all')
    parser.add_argument('-s', '--t_start', metavar='<Y-m-d H-M-S>', default='None', help='Start time in the format Y-m-d H-M-S')
    parser.add_argument('-e', '--t_end', metavar='<Y-m-d H-M-S>', default='None', help='End time in the format Y-m-d H-M-S')

    args = parser.parse_args()

    ok = validate(args.outputs, args.region, args.t_start, args.t_end)
    # If any of the checks fail, exit with error flag, which will be picked up by test framework as a fail
    if not ok:
        sys.exit(1)


if __name__ == "__main__":
    main()



from __future__ import absolute_import
from setuptools import setup, find_packages
from os import path

_dir = path.abspath(path.dirname(__file__))

with open(path.join(_dir, 'ecfas', 'version.py')) as f:
  exec(f.read())

with open(path.join(_dir, 'README.md')) as f:
  long_description = f.read()

setup(name='ecfas',
  version=__version__,
  long_description=long_description,
  long_description_content_type='text/markdown',
  url='https://gitlab.mercator-ocean.fr/mirazoki/ecfas',
  author='Maialen Irazoki',
  author_email='mirazoki@mercator-ocean.fr',
  license='BSD Licence',
  packages=find_packages(exclude=["test", "functional_test"]),
  package_data={
    'ecfas': ['output_points/*.txt', 'fes/*ini'],
  },

  project_urls={
    'Repository': 'https://gitlab.mercator-ocean.fr/mirazoki/ecfas',
  },

  classifiers=[
    'Development Status :: 4 - Beta',
    'Intended Audience :: Science/Research',
    'Topic :: Scientific/Engineering',
    'Environment :: Console',
    'Programming Language :: Python :: 3.8',
    'License :: OSI Approved :: BSD License',
  ],

  install_requires=[
        'cartopy==0.18.0',
        'dask==2021.9.1',
        'matplotlib==3.3.3',
        'scipy==1.5.3',
        'netcdf4==1.5.3',
        'motuclient==1.8.8',
        'configobj==5.0.6'
  ],

  setup_requires=['flake8'],
  tests_require=['pytest', 'pytest_cov'],
  python_requires='>3.8,<3.9',
)



import glob
import logging
import os
from pathlib import Path
import shutil

from configobj import ConfigObj

import datetime as dt
import ecfas.cmems_datasets as dsets
import numpy as np

log = logging.getLogger(__name__)


def clean_dir(dir_path):
    try:
        shutil.rmtree(dir_path)
    except Exception as e:
        log.error('Failed to delete %s. Reason: %s' % (dir, e))


def read_config(config_file):
    """ Reads a standard key=value config file """

    try:
        return ConfigObj(config_file, raise_errors=True, file_error=True)
    except:
        log.error("Error: " + config_file + " not found or unreadable")
        raise


def validate_date(self, s, date_format="%Y%m%d"):
    """ Validates a string representation of a date against the given format """

    try:
        return dt.datetime.strptime(s, date_format).date()
    except ValueError:
        log.error("Not a valid date: '{0}'.".format(s))
        raise


def check_downloads(outdir, region, t_start, t_end, label):

    log.info("Check all necessary CMEMS files have been downloaded")

    datadir = os.path.join(outdir, region, 'data')
    expected_dates = [t_start + dt.timedelta(days=i) for i in np.arange(np.ceil((t_end - t_start).total_seconds() / (86400)))]

    varshyd = []
    varswav = []

    if region in dsets.datasets_hydro.keys():
        varshyd = dsets.datasets_hydro[region]['var']
    if region in dsets.datasets_wave.keys():
        varswav = dsets.datasets_wave[region]['var']

    for vari in varshyd:
        files = glob.glob(os.path.join(datadir, vari + '*' + label + '*.nc'))
        files.sort()
        dateini_list = get_existing_files_start_dates(files)
        missing = [datei for datei in expected_dates if datei not in dateini_list]
        missing.sort()
        if len(missing) > 0:
            for imiss in missing:
                log.error("Missing PHY field data for date %s\n" % imiss.strftime("%Y-%m-%d_%H-%M-%S"))
            return False

        # only check WAV fields for the PHY fields available
        for varj in varswav:
            if isinstance(varj, list):
                files = glob.glob(os.path.join(datadir, 'WAVsubset_' + label + '*.nc'))
            else:
                files = glob.glob(os.path.join(datadir, varj + '_' + label + '*.nc'))
            files.sort()
            dateini_list_wav = get_existing_files_start_dates(files)

            missing = [datei for datei in dateini_list if datei not in dateini_list_wav]
            missing.sort()

            if len(missing) > 0:
                for imiss in missing:
                    log.error("Missing WAV field data for date %s\n" % imiss.strftime("%Y-%m-%d_%H-%M-%S"))
                return False

    return True


def get_existing_files_start_dates(files):

    start_dates = []
    for f in files:
        pathi, fnameext = os.path.split(f)
        fname = os.path.splitext(fnameext)[0]
        _, _, _, dateinia, dateinib, _, _ = fname.split("_")
        start_dates.append(dt.datetime.strptime((dateinia + ' ' + dateinib), '%Y-%m-%d %H-%M-%S'))

    return start_dates


def check_timeseries_output_size(file, exp_size, tolerance):

    log.info("Check timeseries output size")

    if len(file) == 0:
        log.error("File %s is empty!" % file)
        return False
    else:
        filesize = Path(file).stat().st_size
        if not np.isclose(filesize, exp_size, rtol=tolerance):
            log.warning("File %s size %x differs from expected %x" % (file, filesize, exp_size))
            return False
    return True



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Feb  9 12:35:23 2021

@author: mirazoki
validation of forecasted WLs
"""
import datetime
import glob
import os
import pathlib
import sys
import urllib
import logging

import cartopy
from cartopy.mpl.gridliner import LONGITUDE_FORMATTER, LATITUDE_FORMATTER
import matplotlib
from matplotlib.gridspec import GridSpec
from mpl_toolkits.axes_grid1 import make_axes_locatable
from scipy import interpolate
from scipy.interpolate import LinearNDInterpolator
from scipy.interpolate import RegularGridInterpolator
from scipy.spatial import Delaunay
from scipy.spatial import cKDTree as KDTree
from scipy.spatial import cKDTree as KDTree

import cartopy.crs as ccrs
import ecfas.cmems_datasets as dsets
import ecfas.download_insitu_obs as dio
import ecfas.swash_formulations
import matplotlib.pyplot as plt
import netCDF4 as nc
import numpy as np
import pandas as pd
import xarray as xr

log = logging.getLogger(__name__)

# import dask
def coast_water_level_validate(reg, PHYvar, outdir, filename, tini, tend, obstype, getobs=True, t0=None, obsdir=None):
            # coast_water_level_validate(region,PHYvar,datadir,outdir,filename,tini,tend,obstype,t0=tsel[ifile])

    zoom = 2  # deg

    if '15min' in reg:
        regobs = reg.replace('_15min', '')
    elif 'tsm' in reg:
        regobs = reg.replace('_tsm', '')
    else:
        regobs = reg

    if obsdir is None:
        obsdir = os.path.join(r'/homelocal2/mirazoki/ECFAS/data/CMEMS/observations/insitu', regobs)

    if not os.path.exists(outdir):
        os.mkdir(outdir)
    #==========================MODEL============================================
    datamodel = xr.open_dataset(filename)

    if 'lon' in datamodel.variables.keys():
        datamodel = datamodel.rename({'lon':'longitude', 'lat':'latitude'})

    kdmodel = KDTree(np.column_stack((datamodel['longitude'].values, datamodel['latitude'].values)))

    # loop over observations and retrieve closest point, and plot!

    #==========================OBS============================================

    searchvars = [['SWHT', 'VSPEC1D'],
                ['SLEV']]
    # outdirobs=os.path.join(obsdir,reg)
    outdirobs = obsdir
    if getobs:
        dio.download_insitu_obs(regobs, tini, tend, obstype='latest', chosen=['TG'], outdir=outdirobs)

    # read observation list from the index files
    colidx = ['product_id', 'file_name', 'geospatial_lat_min', 'geospatial_lat_max', 'geospatial_lon_min', 'geospatial_lon_max', 'time_coverage_start', 'time_coverage_end', 'institution', 'date_update', 'data_mode', 'parameters']
    id_data = pd.read_csv(os.path.join(obsdir, 'index_%s.txt' % (obstype)), sep=',', delimiter=None, header=None, skiprows=6, names=colidx, index_col=None)
    boolvar_PHY = [np.any([vari in row['parameters'] for vari in searchvars[1]]) for ii, row in id_data.iterrows()]

    # if obstype=='monthly' or obstype=='latest':
    #     #just take the unique station names. take first date array for example
    #     # datestr=pathlib.Path(id_data['file_name'].iloc[0]).parts[-2]
    #     datestr=tini.strftime("%Y%m%d")
    #     #filter those for the chosen times
    #     boolvar_un=[datestr in row['file_name'] for ii,row in id_data.iterrows()]
    #     boolvar_PHY=np.array(boolvar_PHY) & np.array(boolvar_un)

    #     listncs=[pathlib.Path(filei).parts[-1].replace('.nc','') for filei in id_data['file_name'].loc[boolvar_PHY]]
    #     listobs=['_'.join(nci.split('_')[0:-1]) for nci in listncs]
    # else:
    #     listobs=[pathlib.Path(filei).parts[-1].replace('.nc','') for filei in id_data['file_name'].loc[boolvar_PHY]]

    if obstype == 'monthly' or obstype == 'latest':
        # just take the unique station names. take first date array for example
        # datestr=pathlib.Path(id_data['file_name'].iloc[0]).parts[-2]
        datestr = tini.strftime("%Y%m%d")
        # filter those for the chosen times
        boolvar_un = [datestr in row['file_name'] for ii, row in id_data.iterrows()]
        if sum(boolvar_un) == 0:  # we are using a id file that doesn't match what we are looking for'
            # take the first date available, use that as ref
            datestr = datetime.datetime.strptime(id_data['time_coverage_start'].iloc[int(id_data.index.size / 2)], '%Y-%m-%dT%H:%M:%SZ')
            datestr = datetime.datetime(datestr.year, datestr.month, datestr.day).strftime("%Y%m%d")
            boolvar_un = [datestr in row['file_name'] for ii, row in id_data.iterrows()]
        boolvar_PHY = np.array(boolvar_PHY) & np.array(boolvar_un)

        listncs = [pathlib.Path(filei).parts[-1].replace('.nc', '') for filei in id_data['file_name'].loc[boolvar_PHY]]
        listobs = ['_'.join(nci.split('_')[0:-1]) for nci in listncs]
    else:
        listobs = [pathlib.Path(filei).parts[-1].replace('.nc', '') for filei in id_data['file_name'].loc[boolvar_PHY]]

    # listobs=glob.glob(os.path.join(obsdir,obstype,'*TG*.nc')) #these are directories

    for ii, iobs in enumerate(listobs):
        # ncobs=glob.glob(os.path.join(iobs,'*SLEV*.nc'))
        # if len(iobs)>0:
        if obstype == 'latest':
            # concatenate by day
            listfiles = glob.glob(os.path.join(obsdir, obstype, '**', iobs + '*.nc'))
            # listfiles=glob.glob(os.path.join(obsdir,obstype,iobs + '*.nc'), recursive=True)
            listfiles.sort()

            if len(listfiles) > 0:
                # do it manually, not mf dataset
                listxr = []
                for i, filei in enumerate(listfiles):
                    datai = xr.open_dataset(filei, drop_variables=['POSITION', 'POSITION_QC'])
                    datai = datai.isel(LONGITUDE=0, LATITUDE=0, DEPTH=0)
                    listxr.append(datai)
                try:
                    dataobs = xr.concat(listxr, dim='TIME')
                except:
                    log.warning(' Could not manage to concatenate observations. Skip station\n')
                    continue  # go to next station
            else:
                continue  # go to next station

            # dataobs=xr.open_mfdataset(listfiles,concat_dim='TIME',data_vars=['SLEV'],coords='all',compat="override")
        # elif obstype=='monthly':
        #     dataobs=xr.open_dataset(os.path.joiniobs)
        # elif obstype=='history':
        #     dataobs=xr.open_dataset(os.path.joiniobs)
        # CLIP to given timeframe

        # make unique in time
        _, index = np.unique(dataobs['TIME'], return_index=True)
        dataobs = dataobs.isel(TIME=index)
        datasel = dataobs.sel(TIME=slice(tini.strftime("%Y-%m-%d"), tend.strftime("%Y-%m-%d")))
        if datasel.dims['TIME'] > 0:
            log.debug(iobs)

            try:
                # some data found. interpolate model values to the available observed times and geo location
                # dist,i=kdmodel.query(np.array([np.squeeze(datasel.LONGITUDE[0].values),np.squeeze(datasel.LATITUDE[1].values)]),distance_upper_bound=0.25)
                dist, i = kdmodel.query(np.array([np.squeeze(datasel.LONGITUDE.values), np.squeeze(datasel.LATITUDE.values)]), distance_upper_bound=0.25)

                if not np.isinf(dist):
                    datamodeli = datamodel.isel(stations=i)
                    # plot!
                    datesmod = matplotlib.dates.date2num(datamodeli['time'].values)
                    datesobs = matplotlib.dates.date2num(datasel['TIME'].values)

                    # #stats

                    # errarray=mdata_intp[~nanobs]-obsdata.values[~nanobs]
                    # stats=dict()
                    # stats['bias']=np.mean(errarray)
                    # stats['std']=np.std(errarray)
                    # stats['stdobs']=np.std(obsdata.values[~nanobs])
                    # stats['stdmod']=np.std(mdata_intp[~nanobs])
                    # stats['rmse']=np.sqrt(np.square(stats['std'])+np.square(stats['bias']))
                    # rpears=np.corrcoef(obsdata.values[~nanobs],mdata_intp[~nanobs])
                    # stats['rpears']=rpears[0,1]

                    figid = plt.figure(figsize=(12, 12))
                    gs = GridSpec(3, 2, width_ratios=[3, 1])
                    axloc = plt.subplot(gs[:, 1], projection=ccrs.PlateCarree())
                    axloc.add_feature(cartopy.feature.LAND.with_scale('10m'), facecolor='gray', zorder=-1, alpha=1, edgecolor='black')
                    axloc.set_extent([datamodeli.longitude.values - zoom, datamodeli.longitude.values + zoom, datamodeli.latitude.values - zoom, datamodeli.latitude.values + zoom], ccrs.PlateCarree())
                    axloc.plot(datamodeli.longitude.values, datamodeli.latitude.values, color='blue', linewidth=2, marker='o', transform=ccrs.PlateCarree(), label='mod')
                    # axloc.plot(datasel.LONGITUDE[0].values,datasel.LATITUDE[0].values,color='red', linewidth=2, marker='*',transform=ccrs.PlateCarree(),label='obs')
                    axloc.plot(datasel.LONGITUDE.values, datasel.LATITUDE.values, color='red', linewidth=2, marker='*', transform=ccrs.PlateCarree(), label='obs')

                    axloc.legend()

                    # for now, bias correct
                    ax1 = plt.subplot(gs[0, 0])
                    ax1.plot_date(datesobs, datasel['SLEV'].values - np.nanmean(datasel['SLEV'].values), linestyle='--', color='k', marker=None, label='obs', zorder=10)
                    ax1.plot_date(datesmod, datamodeli[PHYvar].values - np.nanmean(datamodeli[PHYvar].values), linestyle='-', color='b', marker=None, label='model', zorder=0)
                    # ax1.plot_date(datesmod,datamodeli['zos'].values-np.mean(datamodeli['zos'].values)+datamodeli['Vsetup'].values,linestyle='-',color='c',marker=None,label='model+setup')
                    # ax1.plot_date(datesmod,datamodeli['zos'].values-np.mean(datamodeli['zos'].values)+datamodeli['VHM0'].values*0.2,linestyle='-',color='g',marker=None,label='model+0.2Hs')
                    # ax1.plot_date(dateswl,datamodeli['zos'].values-np.mean(datamodeli['zos'].values)+datamodeli['Vsetup'].values+datamodeli['VS_ig'].values+datamodeli['VS_inc'].values,linestyle='-')
                    try:
                        ax1.set_ylim([np.floor(np.nanmin(datasel['SLEV'].values - np.nanmean(datasel['SLEV'].values))), np.ceil(np.nanmax(datasel['SLEV'].values - np.nanmean(datasel['SLEV'].values)))])
                    except:
                        try:
                            ax1.set_ylim([np.floor(np.nanmin(datamodeli[PHYvar].values - np.nanmean(datamodeli[PHYvar].values))), np.ceil(np.nanmax(datamodeli[PHYvar].values - np.nanmean(datamodeli[PHYvar].values)))])
                        except:
                            ax1.set_ylim([-1, 1])
                    ax1.xaxis.set_tick_params(rotation=45)
                    ax1.legend(loc='lower right')
                    ax1.set_ylabel('TWL[m]')

                    if 'VHM0' in datamodeli.variables.keys():
                        # for now, bias correct
                        ax2 = plt.subplot(gs[1, 0])
                        ax2.plot_date(datesmod, datamodeli['VHM0'].values, linestyle='-', color='c', marker=None, label='Hs')
                        # ax2.plot_date([matplotlib.dates.date2num(t0),matplotlib.dates.date2num(t0)],[-10,10],'--r')
                        try:
                            ax2.set_ylim([np.floor(np.nanmin(datamodeli['VHM0'].values)), np.ceil(np.nanmax(datamodeli['VHM0'].values))])
                        except:
                            ax2.set_ylim([0, 1])

                        ax2.legend(loc='lower right')
                        ax2.set_ylabel('Hs[m]')
                        ax2.xaxis.set_tick_params(rotation=45)

                        # for now, bias correct
                        ax3 = plt.subplot(gs[2, 0])
                        ax3.plot_date(datesmod, datamodeli['VTPK'].values, linestyle='-', color='g', marker=None, label='Tp')
                        # ax3.plot_date([matplotlib.dates.date2num(t0),matplotlib.dates.date2num(t0)],[-100,100],'--r')
                        try:
                            ax3.set_ylim([np.floor(np.nanmin(datamodeli['VTPK'].values)), np.ceil(np.nanmax(datamodeli['VTPK'].values))])
                        except:
                            ax3.set_ylim([0, 1])
                        ax3.legend(loc='lower right')
                        ax3.set_ylabel('Tp[s]')
                        ax3.set_xlabel('time')
                        ax3.xaxis.set_tick_params(rotation=45)

                    if t0 is not None:
                        ax_list = figid.axes
                        for axi in ax_list:
                            axi.plot_date([matplotlib.dates.date2num(t0), matplotlib.dates.date2num(t0)], [-10, 10], '--r', label='t0')
                            axi.legend(loc='lower right')

                    figid.suptitle(os.path.basename(iobs).split('.')[0])
                    outname = os.path.join(outdir, 'WL_%s_t0%s_%s_%s.png' % (os.path.basename(iobs).split('.')[0], t0.strftime("%Y-%m-%d"), tini.strftime("%Y-%m-%d"), tend.strftime("%Y-%m-%d")))
                    plt.savefig(outname, dpi=100, bbox_inches='tight')
                    plt.close('all')
            except:
                log.warning("=====Something failed in the observation data extraction. Continue\n====")


if __name__ == "__main__":

    trangeval = [datetime.datetime(2021, 3, 1), datetime.datetime(2021, 3, 2)]  # list of t0s
    # trangeval=[]
    listreg = list(dsets.datasets_hydro.keys())
    listreg = ['BAL']

    if len(sys.argv) > 1:
        listreg = [str(sys.argv[1])]
        tstart = datetime.datetime.strptime(sys.argv[2], "%Y%m%d")
        tend = datetime.datetime.strptime(sys.argv[3], "%Y%m%d")
        trangeval = [tstart, tend]  # list of t0s

    datadir = r'/homelocal2/mirazoki/ECFAS/data/CMEMS'  # '/IBI/timeseries/'
    obstype = 'latest'  # latest,monthly,history
    for region in listreg:
        PHYvar = dsets.datasets_hydro[region]['var'][0]

        # identify all t0 that haven't been made into timeseries
        listzos = glob.glob(os.path.join(datadir, region, 'timeseries', 'tseries_coastal_*.nc'))
        listzos.sort()
        t0list = []
        for filei in listzos:
            parts = os.path.split(filei)[-1].split('_')
            t0list.append(datetime.datetime.strptime(parts[2] + '_' + parts[3], "%Y-%m-%d_%H-%M-%S"))
        t0list, idxu = np.unique(t0list, return_index=True)
        listzos = np.array(listzos)[idxu]
        if len(trangeval) > 0:
            # clip the t0s to the given range
            boolsel = [t0i > trangeval[0] and t0i < trangeval[1] for t0i in t0list]
        else:
            boolsel = np.ones(listzos.shape, dtype='bool')

        tsel = t0list[np.array(boolsel)]
        filesel = listzos[np.array(boolsel)]
        for ifile, filename in enumerate(filesel):
            parts = os.path.split(filename)[-1].split('_')
            tini = datetime.datetime.strptime(parts[4] + '_' + parts[5], "%Y-%m-%d_%H-%M-%S")
            tend = datetime.datetime.strptime(parts[6] + '_' + parts[7].replace('.nc', ''), "%Y-%m-%d_%H-%M-%S")
            outdir = os.path.join(os.path.split(filename)[0], 'figures')
            coast_water_level_validate(region, PHYvar, outdir, filename, tini, tend, obstype, t0=tsel[ifile])



"""
@author: mirazoki, gturek
operational download of CMEMS data
"""
import argparse
from distutils.log import debug
import logging
import os
import shutil
import sys
import time

import datetime as dt
import ecfas.cmems_datasets as dsets
import ecfas.coast_water_level_extract_multiple as cem
import ecfas.coast_water_level_process as cp
import ecfas.fes.module_FES2014 as fes
import ecfas.motu_download as md
import ecfas.utils as utils
import logging.handlers as handlers


def init_logger(log_dir):
    """ Initialize logging """

    log_level = logging.INFO
    if debug:
        log_level = logging.DEBUG
    logging.basicConfig(format='%(asctime)s:%(name)s:%(levelname)-4s:%(message)s', level=log_level, datefmt='%Y-%m-%d %H:%M:%S')

    global log
    log = logging.getLogger('ecflow')
    log.addHandler(logging.StreamHandler())

    if log_dir != "":
        os.makedirs(log_dir, exist_ok=True)
        log_dir = os.path.join(log_dir, 'ecflow.log')
        # Rotate log daily into a new file up to a maximum number of 2 backups
        log.info(f'Logging to file {log_dir}')
        file_handler = handlers.TimedRotatingFileHandler(log_dir, when='midnight', interval=1, backupCount=3)
        file_handler.setFormatter(logging.Formatter("%(asctime)s:%(name)s:%(levelname)-4s:%(message)s", "%Y-%m-%d %H:%M:%S"))
        file_handler.setLevel(log_level)
        log.addHandler(file_handler)


def clean_working_dirs(data_sets, out_dir, region):
    """ Re-initializes all 'data' directories at the beginning of each run """
    setsel = []

    if region is None:
        setsel.append(list(data_sets.keys()))
    elif region == 'ARC':
        setsel.append('ARC_ocean')
        setsel.append('ARC')
    else:
        setsel.append(region)

    for dseti in setsel:
        out = os.path.join(out_dir, '%s' % dseti)
        out = os.path.join(out, 'data')

        # Clean any pre-existing data files
        if os.path.exists(out):
            shutil.rmtree(out)

        os.makedirs(out)


def get_datasets(dset_type, mod_type):
    if mod_type == 'fcst':
        if dset_type == 'PHY':
            data_sets = dsets.datasets_hydro
        elif dset_type == 'WAV':
            data_sets = dsets.datasets_wave
    elif mod_type == 'reanal':
        if dset_type == 'PHY':
            data_sets = dsets.datasets_hydro_reanal
        elif dset_type == 'WAV':
            data_sets = dsets.datasets_wave_reanal
    return data_sets


def execute_workflow(args):

    region = args.region
    mod_type = 'fcst'
    if args.reanal:
        mod_type = 'reanal'
    global debug
    debug = args.debug

    # Read config file
    try:
        config = utils.read_config(args.config)
    except Exception as e:
        print('Failed to read config file: ' + str(e))
        sys.exit(1)

    outdir = config['outdir']
    maskdir = config['maskdir']
    logdir = config['logdir']
    fesdir = config['fesdir']
    # FES code will read env variable $FES_DATA to find where the data lives
    os.environ['FES_DATA'] = fesdir
    user = config['usr']
    pwd = config['pwd']
    motu = config['url']

    init_logger(config['logdir'])

    if args.t0 == None:
        uptime = dsets.datasets_hydro[region]["uptime"]
        now = dt.datetime.now()
        next_start = dt.datetime(now.year, now.month, now.day, int(uptime), int((uptime * 60) % 60), int((uptime * 3600) % 3600))
    else:
        next_start = dt.datetime.strptime(args.t0, "%Y%m%d_%H%M%S")

    #1. DOWNLOAD THE FIELDS ==========================================================================================================
    log.info(">> Downloading data from CMEMS")
    pred_start = next_start - dt.timedelta(days=2)
    pred_end = next_start + dt.timedelta(days=5)

    # call function
    t_start = pred_start.strftime("%Y-%m-%d %H:%M:%S")
    t_end = pred_end.strftime("%Y-%m-%d %H:%M:%S")
    label = next_start.strftime("%Y-%m-%d_%H-%M-%S")

    tic_download = time.time()
    datadir = os.path.join(outdir, '%s' % region)

    data_sets_phy = get_datasets('PHY', mod_type)
    if not debug:
        clean_working_dirs(data_sets_phy, outdir, region)
    data_sets_wav = get_datasets('WAV', mod_type)
    tries = 1
    while tries <= 2:
        md.motu_download(user, pwd, motu, t_start, t_end, outdir, data_sets_phy, 'PHY', region=region, modtype=mod_type, label=label)
        md.motu_download(user, pwd, motu, t_start, t_end, outdir, data_sets_wav, 'WAV', region=region, modtype=mod_type, label=label)
        ok = utils.check_downloads(outdir, region, pred_start, pred_end, label)
        if ok:
            break
        else:
            tries += 1
            if tries > 2:
                log.error("Problems downloading CMEMS data, please check server status")
                sys.exit(1)
            log.warning("Retrying download, some files were found to be missing")

    toc_download = time.time()

    #2. CLIP TO COASTAL LEVELS ========================================================================================================
    log.info(">> Clipping to coastal levels")
    tic_clip = time.time()

    outdirts = os.path.join(datadir, 'data')
    if region not in dsets.datasets_wave:
        add_wave = False
    else:
        add_wave = True
    # clip to coastal points
    if region == 'ARC':
        cem.coast_water_level_extract_multiple(datadir.replace(region, 'ARC_ocean'), outdirts.replace(region, 'ARC_ocean'), maskdir, 'ARC_ocean', next_start.strftime("%Y-%m-%d_%H-%M-%S"), pred_start, pred_end, False)
    cem.coast_water_level_extract_multiple(datadir, outdirts, maskdir, region, next_start.strftime("%Y-%m-%d_%H-%M-%S"), pred_start, pred_end, add_wave)
    toc_clip = time.time()

    #3. COMBINE DATASETS AND CALCULATE SETUP ===========================================================================================
    log.info(">> Combining datasets and calculating setup")
    tic_process = time.time()
    tide = None

    outdirproc = os.path.join(datadir, 'timeseries')
    if region in ['BS']:
        # initialize FES
        short_tide, radial_tide = fes.init_fes_h(fesdir)
        tide = dict()
        tide['short_tide'] = short_tide
        tide['radial_tide'] = radial_tide
        tide['fesdir'] = fesdir
    cp.coast_water_level_process(region, outdirts, outdirproc, label,maskdir, swashtype='cte', betatype='None', fes_tide=tide)
    toc_process = time.time()

    outfile = os.path.join(datadir, 'time_diagnostics.txt')
    file = open(outfile, 'w')
    file.write('Time download CMEMS: %s\n' % (toc_download - tic_download))
    file.write('Time snip CMEMS: %s\n' % (toc_clip - tic_clip))
    file.write('Time process CMEMS: %s\n' % (toc_process - tic_process))
    file.close()


def main():
    #0. READ ARGS AND CONFIG ==========================================================================================================
    parser = argparse.ArgumentParser("op_workflow")
    parser.add_argument('-c', '--config', metavar='<config_file>', default='ecfas.cfg', required=True, help='Absolute path to config file')
    parser.add_argument('-r', '--region', metavar='<region>', default=None, help='Region of interest, one of ARC, BAL, BS. IBI, MED, NWS, GLO. Defaults to all')
    parser.add_argument('-t', '--t0', metavar='<YYmmdd_HHMMSS>', default=None, help='Start time t0 in the format YYmmdd_HHMMSS')
    parser.add_argument('--reanal', action='store_true', help='reanal mode')
    parser.add_argument('--debug', action='store_true', help='Debug mode')

    execute_workflow(parser.parse_args())


if __name__ == "__main__":
    main()
